import gtk
import gobject
import dbus
import os
import signal
import time

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

class LeakWindow:

    def __init__ (self):
        self.timer_id = 0
        self.got_reply = True
        self.count_info = {}
        self.window = gtk.Window ()
        self.window.set_default_size (450, 500)
        self.window.connect("delete-event", gtk.main_quit)

        alignment = gtk.Alignment (0.5, 0.5, 1.0, 1.0)
        alignment.set_padding(12, 12, 12, 12)

        self.vbox = gtk.VBox(False, 12)
        alignment.add (self.vbox)
        self.window.add (alignment)

        self.hbox1 = gtk.HBox(False, 6)

        pid_label = gtk.Label("PID:")
        self.hbox1.add (pid_label)
        self.pid_entry = gtk.Entry()
        self.hbox1.add (self.pid_entry)

        snapshot_button = gtk.Button()
        snapshot_button.set_label ("Snapshot")
        snapshot_button.connect ("clicked", self.send_signal)
        self.hbox1.add(snapshot_button)

        self.periodic_cb = gtk.CheckButton()
        self.periodic_cb.set_label("Periodic updates")
        self.periodic_cb.connect("toggled", self.toggle_periodic)
        self.hbox1.add (self.periodic_cb)

        self.vbox.pack_start (self.hbox1, False, True)

        # timestamp
        self.time_label = gtk.Label ()
        self.time_label.set_alignment (1.0, 0.5)
        self.vbox.pack_start (self.time_label, False, True)

        # main view
        self.model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_UINT, gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)
        scrolled_window = gtk.ScrolledWindow()
        self.treeview = gtk.TreeView (self.model)
        text_renderer = gtk.CellRendererText ()
        name_col = gtk.TreeViewColumn("Type name", text_renderer, text=0)
        name_col.set_sort_column_id(0)
        self.treeview.append_column(name_col)
        text_renderer = gtk.CellRendererText ()
        num_col = gtk.TreeViewColumn("Instance count", text_renderer, text=1)
        num_col.set_sort_column_id(1)
        num_col.set_sort_order(gtk.SORT_DESCENDING)
        self.treeview.append_column(num_col)
        text_renderer = gtk.CellRendererText ()
        delta_col = gtk.TreeViewColumn("delta", text_renderer, text=3, background_set=4, background=5)
        delta_col.set_sort_column_id(2)
        self.treeview.append_column(delta_col)
        self.treeview.set_search_column(1)
        scrolled_window.add(self.treeview)
        self.vbox.add (scrolled_window)

        self.window.show_all ()

    def toggle_periodic(self, check_btn):
        if check_btn.get_active() and self.timer_id == 0:
            self.timer_id = gobject.timeout_add (2000, self.signal_timeout)
        elif not check_btn.get_active() and self.timer_id != 0:
            gobject.source_remove(self.timer_id)
            self.timer_id = 0

    def signal_timeout(self):
        if not self.got_reply:
            self.timer_id = 0
            self.periodic_cb.set_active(False)
            return False

        self.send_signal ()

        return True

    def send_signal(self, button = None):
        pid = int(self.pid_entry.get_text ())
        self.got_reply = False
        os.kill(pid, signal.SIGUSR1)

    def dbus_signal (self, *args):
        pid, instances = args
        self.model.clear ()
        self.got_reply = True
        for info in instances:
            name, count = info
            last_cnt = self.count_info[name] if name in self.count_info else 0
            self.count_info[name] = count
            delta = count - last_cnt
            delta_str = str(count - last_cnt) if delta != 0 else ""
            colorize = False
            color = None
            if delta_str != "":
                colorize = delta != 0
                if delta > 0:
                    delta_str = '+' + delta_str
                    color = 'red'
                elif delta_str[0] == '-':
                    color = 'green'
            self.model.append ([name, count, delta, delta_str, colorize, color])

        self.time_label.set_text ("Last update:" + time.strftime("%H:%M:%S"))

if __name__ == "__main__":
    bus = dbus.SessionBus ()
    w = LeakWindow ()
    bus.add_signal_receiver(w.dbus_signal, "ObjectCountsSnapshot", "org.debug.LeakDetector", None, None)
    gtk.main ()
